from __future__ import with_statement

import xml.etree.ElementTree as ET
from datetime import datetime
import hashlib
import os, posixpath
import subprocess
import sys
import tempfile

from . import http


class Updater(object):
    def __init__(self, url='', build_date='', **kwargs):
        self.__dict__.update(kwargs)
        self.url = posixpath.join(url, sys.platform + '.xml')
        self.build_date = build_date
        self.__xml = None
        self.__info = None
        
    def has_update(self):
        xml = self.get_xml()
        date = xml.find('build-date')
        if date is not None:
            date = date.text
            return date > self.build_date
        return False
        
    def get_info(self):
        xml = self.get_xml()
        if self.__info is None:
            info = {}
            node = xml.find('build-date')
            if node is not None: info['build-date'] = node.text
            node = xml.find('version')
            if node is not None: info['version'] = node.text
            node = xml.find('title')
            if node is not None: info['title'] = node.text
            node = xml.find('description')
            if node is not None:
                if node.get('type') == 'text/html':
                    info['description'] = ET.tostring(node).replace('description>', 'div>')
                else:
                    info['description'] = node.text
            info['links'] = {}
            for node in xml.findall('link'):
                info['links'][node.get('rel')] = node.get('href')
            info['messages'] = []
            for node in xml.findall('message'):
                info['messages'].append(node.text)
            self.__info = info
        return self.__info
        
    def can_autoupdate(self):
        info = self.get_info()
        return 'update' in info['links']
        
    def get_homepage_url(self):
        info = self.get_info()
        return info['links'].get('homepage', None)
        
    def begin_update(self, onprogress_callback, oncomplete_callback):
        info = self.get_info()
        if 'update' in info['links']:
            return self.__download(info['links']['update'], onprogress_callback, oncomplete_callback)
        return False
        
    def __download(self, url, onprogress_callback, oncomplete_callback):
        response = http.request(url, progress_callback=onprogress_callback)
        if response['code'] == 200:
            fn = posixpath.basename(url)
            if not fn: fn = hashlib.md5(url).hexdigest()
            fn = os.path.join(tempfile.gettempdir(), fn)
            with open(fn, 'wb') as f:
                f.write(response['body'])
            oncomplete_callback(fn)
            return fn
        else:
            raise IOError("URL \"%s\" not retrieved - status %s" % (url, response['code']))
            
    def get_xml(self):
        if self.__xml is None:
            response = http.request(self.url)
            if response['code'] == 200:
                self.__xml = ET.XML(response['body'])
            else:
                raise IOError("URL \"%s\" not retrieved - status %s" % (self.url, response['code']))
        return self.__xml


def install(filename):
    if hasattr(sys, 'frozen'):
        if sys.platform == 'win32':
            subprocess.Popen([filename])
        elif sys.platform == 'darwin':
            app_path = os.path.abspath(sys.argv[0])
            app_dir = os.path.dirname(app_path)
            subprocess.Popen([os.path.join(app_dir, 'do_update.sh'), app_path, filename])
    else:
        print 'Not updating non-frozen app'
    sys.exit(0)


