from __future__ import with_statement

import os

import wx
import wx.lib.sized_controls as sc
import wx.gizmos
import HyperTreeList

import AppInfo
from Config import Config, ParseDict
import Command
import SshCommand
import Connection
import Images
import Location
import Plugins
import Window
import controls
from . import messages
from . import dialog

class Tool(Window.Tool):
    can_toolbar = True
    can_menu = True


class Export(Tool):
    name = 'export'
    label = 'Export'
    description = 'Export bookmarks'
    can_toolbar = False
    def IsAble(self):
        return len(Config.bookmarks) > 0
    def run(self):
        if len(Config.bookmarks) > 0:
            path = dialog.FileSave(
                self.parent, message=self.description,
                directory=os.getcwd(), 
                filename="",
                wildcards=[('JSON Files', '*.json'), ('All Files', '*.*')]
            )
            if path is not None:
                if path[-5:].lower() != '.json':
                    path += '.json'
                Config.save(path)
        else:
            wx.MessageDialog(self.parent, "No bookmarks to export!", 'Export', wx.OK|wx.ICON_ERROR).ShowModal()


class Import(Tool):
    name = 'import'
    label = 'Import'
    description = 'Import bookmarks'
    can_toolbar = False
    def IsAble(self):
        return True
    def run(self):
        path = dialog.FileOpen(
            self.parent, message=self.description,
            directory=os.getcwd(), 
            filename="",
            wildcards=[('JSON Files', '*.json'), ('All Files', '*.*')]
        )
        if path is not None:
            import simplejson as json
            with file(path, 'rt') as f:
                input = ParseDict(json.load(f))
                yes_all = False
                no_all = False
                for k in input.bookmarks:
                    if k in Config.bookmarks:
                        existing = Location.Location(**Config.bookmarks[k])
                        if no_all: continue
                        if not yes_all:
                            r = messages.YesNoAllCancel(self.parent, 'Do you want to replace the existing bookmark to\n%s?' % (existing.getUrl(),), 'Bookmark Exists').ShowModal()
                            if r == messages.CANCEL: break
                            if r == messages.NO: continue
                            if r == messages.NO_ALL:
                                no_all = True
                                continue
                            if r == messages.YES_ALL:
                                yes_all = True
                    Config.bookmarks[k] = input.bookmarks[k]
                if 'keychain' in input:
                    if 'keychain' in Config:
                        Config['keychain'].merge_recursive(input.keychain)
                    else:
                        Config['keychain'] = input.keychain
class Edit(Tool):
    name = 'edit'
    label = 'Edit'
    description = 'Edit selected bookmark'
    default_shortcut = 'Ctrl-E'
    def IsAble(self):
        item = self.parent.GetCurrentUrl()
        return item is not None and item.isBookmarked()
    def run(self, item=None):
        if item is None:
            item = self.parent.GetCurrentUrl()
        if item:
            wx.PostEvent(self.parent, Window.PanelChangeEvent(Editor, bookmark=item))
        else:
            wx.MessageDialog(self.parent, "No bookmark to edit!", 'Edit', wx.OK|wx.ICON_ERROR).ShowModal()

class Add(Tool):
    name = 'add'
    label = 'Add'
    description = 'Add bookmark'
    default_shortcut = 'Ctrl-+'
    def IsAble(self):
        return True
    def run(self, item=None):
        if item is None:
            item = self.parent.GetCurrentUrl()
        if item and item.isBookmarked(): item = None
        wx.PostEvent(self.parent, Window.PanelChangeEvent(Editor, bookmark=item))

class Delete(Tool):
    name = 'delete'
    label = 'Delete'
    description = 'Delete selected bookmark'
    default_shortcut = 'Del'
    def IsAble(self):
        items = self.parent.GetCurrentBookmarks()
        return items is not None and len([item for item in items if item.isBookmarked()]) > 0
    def run(self, items=None):
        if items is None:
            items = self.parent.GetCurrentBookmarks()
        for item in items:
            if item.isBookmarked():
                if wx.ID_YES == wx.MessageDialog(self.parent, "Are you sure you want to delete \"%s\"?" % (item['label'],), "Delete Bookmark", wx.YES_NO|wx.ICON_QUESTION).ShowModal():
                    item.removeBookmark()
        self.parent.Populate()

class Mount(Tool):
    name = 'open'
    label = 'Open'
    description = 'Open selected bookmark'
    default_shortcut = 'Ctrl-O'
    def IsAble(self):
        items = self.parent.GetCurrentBookmarks()
        return items is not None and len(items) > 0
    def run(self, items=None):
        if items is None:
            items = self.parent.GetCurrentBookmarks()
        if len(items):
            self.parent.OpenBookmarks(items)
        else:
            wx.MessageDialog(self.parent, "No bookmark to open!", 'Open', wx.OK|wx.ICON_ERROR).ShowModal()

class Browser(Window.ConfiguredPanel):
    def __init__(self, parent):
        Window.ConfiguredPanel.__init__(self, parent)
        
    def GetCurrentConfig(self):
        config = Window.ConfiguredPanel.GetCurrentConfig(self)
        config.current_tag = self.current_tag
        return config
        
    def CreatePanel(self, parent):
        self.current_tag = self.settings.get('current_tag', 'Recently Used')
        self.tags = controls.ListCtrl(self, multiple_select=False, sort=False)
        self.tags.InsertColumn(0, '')
        self.PopulateTags()
        self.list = controls.ListCtrl(self)
        self.list.InsertColumn(0, 'Name')
        self.list.InsertColumn(1, 'URL')
        self.list.InsertColumn(2, 'Last Accessed')
        font = wx.SystemSettings_GetFont(wx.SYS_DEFAULT_GUI_FONT)
        self.tags.SetFont(font)
        self.list.SetFont(font)
        hsizer = controls.sizers.HBox()
        hsizer.AddWidget(self.tags, 1)
        hsizer.AddWidget(self.list, 3)
        self.tags.On(self.tags.EVT_ITEM_SELECT, self.OnSelectTag)
        self.Populate(self.current_tag)
        return hsizer
        
    def ConnectEvents(self, OnSelectBookmark, OnDeselectBookmark, OnMount):
        self.list.On(self.list.EVT_ITEM_SELECT, OnSelectBookmark)
        self.list.On(self.list.EVT_ITEM_DESELECT, OnDeselectBookmark)
        self.list.On(self.list.EVT_ITEM_DCLICK, OnMount)
        
    def Activate(self):
        self.tags.SetColumnWidth(0, self.tags.GetSize()[0] - 25)
        w = self.list.GetSize()[0]
        self.list.SetColumnWidth(0, self.list.AUTOSIZE)
        self.list.SetColumnWidth(2, self.list.AUTOSIZE)
        self.list.SetColumnWidth(1, w - self.list.GetColumnWidth(0) - self.list.GetColumnWidth(2) - 25)
        
    def UnselectAll(self):
        self.list.UnselectAll()
        
    def OnSelectTag(self, e):
        tag = e.GetText()
        self.Populate(tag)
    
    def GetSelections(self):
        return [data[1] for data in self.list.iterSelectedData()]
        
    def PopulateTags(self):
        systags = ['Recently Used', 'All']
        if len(Config.getUntaggedBookmarks()):
            systags.append('Untagged')
        tags = systags + Config.getAllBookmarkTags()
        for tag in tags:
            i = self.tags.InsertStringItem(self.tags.GetItemCount(), tag)
            self.tags.itemDataMap[i] = [tag]
            if tag == self.current_tag:
                self.tags.SelectItem(i)
                
    def Populate(self, tag=None):
        if tag is None:
            self.PopulateTags()
        else:
            self.current_tag = tag
        self.list.DeleteAllItems()
        if tag == 'Recently Used':
            items = [Location.Location(**Config.bookmarks[id]) for id in Config.bookmarks if Config.bookmarks[id].get('atime', None)]
            if len(items):
                items.sort(Location.cmp_atime)
        elif tag == 'All':
            items = [Location.Location(**bm) for bm in Config.getAllBookmarks()]
        elif tag == 'Untagged':
            items = [Location.Location(**bm) for bm in Config.getUntaggedBookmarks()]
        else:
            items = [Location.Location(**bm) for bm in Config.getTaggedBookmarks(tag)]
        if len(items):
            for bm in items:
                id = self.list.InsertStringItem(self.list.GetItemCount(), bm['label'])
                self.list.SetStringItem(id, 1, Connection.GetUrl(bm))
                self.list.SetStringItem(id, 2, bm.get_atime())
                self.list.SetItemData(id, id)
                self.list.itemDataMap[id] = [bm['label'], Connection.GetUrl(bm), bm.get('atime', '')]
        self.Activate()


class Panel(Window.PanelWithTools):
    title = 'Bookmarks'
    def __init__(self, parent, initial_url='sftp://', location=None):
        self.initial_url = initial_url
        self.item = None
        self.used_shortcuts = {}
        self.menubar = None
        self.menus = {}
        Window.PanelWithTools.__init__(self, parent)
        self.tb.DoLayout()
        self.PostEvent(Window.PageTitleEvent(page=self, title=self.title))
        self.PostEvent(Window.SetupCompleteEvent())
        wx.CallAfter(self.FocusUrl)
        
    def Activate(self):
        self.browser.Activate()
        
    def SaveState(self):
        self.browser.SaveState()
        Window.PanelWithTools.SaveState(self)
        
    def CreatePanel(self, parent):
        panel = Window.Panel(parent)
        vsizer = wx.BoxSizer(wx.VERTICAL)
        panel.SetBackground(Images.GetColor('toolbar', 'background'))
        self.url = wx.TextCtrl(panel, -1, self.initial_url, style=wx.TE_PROCESS_ENTER)
        self.urlpanel_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.urlpanel_sizer.Add(self.url, 1, wx.EXPAND|wx.ALL, 4)
        
        vsizer.Add(self.urlpanel_sizer, 0, wx.EXPAND|wx.ALL)
        vsizer.Add((4,2), 0)
        self.url.Bind(wx.EVT_TEXT, self.OnUrlChange)
        self.url.Bind(wx.EVT_KEY_UP, self.OnUrlKey)
        
        self.browser = Browser(panel)
        self.browser.ConnectEvents(self.OnSelectBookmark, self.OnDeselectBookmark, self.OnMount)
        
        vsizer.Add(self.browser, 1, wx.EXPAND|wx.ALL)
        
        panel.SetSizer(vsizer)
        return panel
        
    def SetupToolbar(self):
        self.tools = [Import(self), Export(self), Window.ToolSeparator(self), Add(self), Edit(self), Delete(self), Mount(self)]
        
    def Populate(self):
        self.browser.Populate()
        
    def UpdateTools(self):
        for tool in self.tools:
            enable = tool.IsAble()
            if tool.can_toolbar:
                self.tb.Enable(tool.name, enable)
            if hasattr(tool, 'menu_item'):
                self.menus['bookmarks'].Enable(tool.menu_item.GetId(), enable)
    
    def UpdateMenu(self, mb):
        self.menubar = mb
        self.menus = {}
        shortcuts = Config.getShortcuts()
        insert_pos = mb.GetMenuCount() - 1
        menu = wx.Menu()
        self.menubar.Insert(insert_pos, menu, 'Bookmarks')
        self.menus['bookmarks'] = menu
        for tool in self.tools:
            if isinstance(tool, Window.ToolSeparator):
                menu.AppendSeparator()
            elif tool.can_menu:
                label = tool.label
                sc = shortcuts.get('bookmarks.' + tool.name, tool.default_shortcut)
                if sc != '' and sc not in self.used_shortcuts:
                    self.used_shortcuts[sc] = tool
                    label += "\t" + sc
                tool.menu_item = menu.Append(-1, label, tool.description)
                wx.EVT_MENU(self.GetGrandParent().GetParent(), tool.menu_item.GetId(), tool.on_trigger)
    
    # def SelectAll(self):
    #     if self.FindFocus() is self.url: return False
    #     self.bookmarks.SelectAll()
    #     return True
    
    def ClipboardCopy(self):
        if self.FindFocus() is self.url: return False
        return Window.PanelWithTools.DoClipboardCopy(self)
    
    def GetClipboardData(self):
        return "\n".join(self.browser.GetSelections())
        
    def GetUrl(self):
        return self.url.GetValue()
        
    def FocusUrl(self):
        self.url.SetFocus()
        self.url.SetInsertionPointEnd()
        
    def GetCurrentUrl(self):
        return Location.ParseUrl(self.url.GetValue())
        
    def GetCurrentBookmarks(self):
        items = self.browser.GetSelections()
        if len(items):
            return [Location.ParseUrl(item) for item in items]
        # else:
        #     return [Location.ParseUrl(self.url.GetValue())]
        return None

    def OnSelectBookmark(self, e):
        self.url.ChangeValue(self.browser.list.itemDataMap[e.GetIndex()][1])
        self.UpdateTools()

    def OnDeselectBookmark(self, e):
        self.UpdateTools()

    def OnUrlChange(self, e):
        self.browser.UnselectAll()
        self.UpdateTools()
        
    def OnUrlKey(self, e=None):
        if e.GetKeyCode() == 13:
            bm = Location.ParseUrl(self.url.GetValue())
            if bm: self.OpenBookmarks([bm])
        e.Skip()
        
    def AddBookmark(self):
        bm = self.GetCurrentUrl()
        if bm and bm.isBookmarked(): bm = None
        wx.PostEvent(self, Window.PanelChangeEvent(Editor, bookmark=bm))
    
    def EditBookmark(self):
        bm = self.GetCurrentUrl()
        if bm:
            wx.PostEvent(self, Window.PanelChangeEvent(Editor, bookmark=bm))
    
    def DeleteBookmark(self):
        bms = self.GetCurrentBookmarks()
        for bm in bms:
            if bm and bm.isBookmarked() and wx.MessageDialog(self, "Are you sure you want to delete \"%s\"?" % (bm['label'],), "Delete Bookmark", wx.YES_NO|wx.ICON_QUESTION).ShowModal() == wx.ID_YES:
                bm.removeBookmark()
        self.browser.Populate()
    
    def OnMount(self, e=None):
        bm = self.GetCurrentBookmarks()
        if bm: self.OpenBookmarks(bm)
        
    def OpenBookmarks(self, locations):
        from lib.platform import keychain # needs importing in UI thread
        location = locations[0]
        module = location.importModule()
        self.PostEvent(Window.PanelChangeEvent(module.Panel, location=location, newtab=False))
        for location in locations[1:]:
            module = location.importModule()
            self.PostEvent(Window.PanelChangeEvent(module.Panel, location=location, newtab=True))
        
    def OnAsyncStart(self, e):
        pass
    def OnAsyncComplete(self, e):
        e.execute()
        
    def GetCurrentConfig(self):
        return {'expandstate':self._GetExpandState()}
    
    def _GetExpandState(self):
        expandstate = {}
        return expandstate
    def OnItemExpanded(self, e):
        pass


class Editor(Window.PanelWithTools):
    def __init__(self, parent, bookmark=None, nextclass=Panel):
        if bookmark is None: bookmark = Location.Location()
        self.nextclass = nextclass
        self.bookmark = bookmark
        self.handlers = []
        Window.PanelWithTools.__init__(self, parent)
        self.PostEvent(Window.SetupCompleteEvent())
        
    def CreatePanel(self, parent):
        
        self.nb = wx.Notebook(parent, style=wx.SUNKEN_BORDER)
        
        if self.bookmark and self.bookmark['type']: self.CreateEditor(self.bookmark['type'])
        else: self.CreateEditor()
        
        return self.nb
        
    def CreateEditor(self, type=''):
        form = sc.SizedPanel(self.nb, -1)
        form.SetSizerType('form')
        
        for child in form.GetChildren():
            form.RemoveChild(child)
        
        wx.StaticText(form, wx.NewId(), 'Connection Type:')
        choices = [getattr(Connection, C).label for C in Connection.__all__]
        if type == '': type = choices[0]
        self.cbType = wx.ComboBox(form, wx.NewId(), type, choices=choices, style=wx.CB_DROPDOWN|wx.CB_READONLY)
        
        self.cbType.Bind(wx.EVT_COMBOBOX, self.OnChangeType)
        
        wx.StaticText(form, wx.NewId(), 'Title:')
        self.tbLabel = wx.TextCtrl(form, wx.NewId())
        self.tbLabel.SetMinSize((200, self.tbLabel.GetSize()[1]))
        self.tbLabel.SetSizerProps(expand=True)
        
        wx.StaticText(form, wx.NewId(), 'Tags:')
        self.tbTags = wx.TextCtrl(form, wx.NewId())
        self.tbTags.SetMinSize((200, self.tbTags.GetSize()[1]))
        self.tbTags.SetSizerProps(expand=True)
        
        self.editor = getattr(Connection, type).BookmarkEditor(form, self.bookmark)
        
        form.SetMinSize(form.GetSize())
        
        self.nb.AddPage(form, 'General')
        
        plugins = [getattr(Plugins, C) for C in Plugins.__all__ if hasattr(getattr(Plugins, C), 'BookmarkEditor')]
        for C in plugins:
            label = C.label
            name = C.shortName
            
            form = sc.SizedPanel(self.nb, -1)
            form.SetSizerType('form')
            
            self.handlers.append(C.BookmarkEditor(form, Config.preferences.get(name, {})))
            self.nb.AddPage(form, label)
            
        if self.bookmark is not None: self.SetupEditor(self.bookmark)
        
        self.tbLabel.SetFocus()
        return form
    def SetupToolbar(self):
        self.tb.Add(self.OnOK, 'save', 'Save', Images.GetIcon('ok'), 'Save Changes')
        self.tb.Add(self.OnCancel, 'cancel', 'Cancel', Images.GetIcon('cancel'), 'Cancel Changes')
    def OnChangeType(self, e):
        bm = self.GetBookmark()
        wx.PostEvent(self, Window.PanelChangeEvent(Editor, bookmark=bm, nextclass=self.nextclass))
    def OnOK(self):
        id = self.bookmark.id
        bm = self.GetBookmark()
        if self.ValidateBookmark(bm):
            if id in Config.bookmarks:
                bm.atime = Config.bookmarks[id].get('atime', None)
                del Config.bookmarks[id]
            if hasattr(self.editor, 'SaveBookmark'):
                self.editor.SaveBookmark(bm)
            for p in self.handlers:
                if hasattr(p, 'SaveBookmark'):
                    p.SaveBookmark(bm)
            bm.saveBookmark()
            wx.PostEvent(self, Window.PanelChangeEvent(self.nextclass, location=bm))
    def OnCancel(self):
        wx.PostEvent(self, Window.PanelChangeEvent(self.nextclass, location=self.bookmark))
    def SetupEditor(self, bookmark):
        self.tbLabel.SetValue(bookmark.get('label', ''))
        if 'tags' in bookmark: self.tbTags.SetValue(', '.join(bookmark.get('tags', '')))
        else: self.tbTags.SetValue('')
        self.editor.SetupEditor(bookmark)
        for p in self.handlers:
            p.SetupEditor(bookmark)
    def GetBookmark(self):
        self.bookmark = Location.Location()
        self.bookmark['label'] = self.tbLabel.GetValue().strip()
        self.bookmark['type'] = self.cbType.GetValue()
        self.bookmark['tags'] = [x.strip() for x in self.tbTags.GetValue().strip().split(',') if x.strip() != '']
        self.editor.UpdateBookmark(self.bookmark)
        for p in self.handlers:
            p.UpdateBookmark(self.bookmark)
        return self.bookmark
    def ValidateBookmark(self, bm):
        errors = []
        valid = True
        if bm['label'] == '': errors.append("Please enter a Label")
        self.editor.ValidateBookmark(bm, errors)
        for p in self.handlers:
            valid &= p.ValidateBookmark(bm, errors)
        if len(errors):
            wx.MessageDialog(self, "\n".join(errors), "Save Bookmark", wx.OK|wx.ICON_ERROR).ShowModal()
            return False
        return valid
