import threading

import wx
import wx.lib.scrolledpanel as scrolled

from lib import logger

import Images

from . import Window, controls

class Transfer(controls.sizers.HBox):
    def __init__(self, parent, transaction):
        controls.sizers.HBox.__init__(self)
        font = Window.GetFont('small')
        self.alive = True
        vsizer = controls.sizers.VBox()
        self._message = controls.Label(parent, transaction.title)
        vsizer.AddWidget(self._message)
        self._text = controls.Label(parent, transaction.text)
        vsizer.AddWidget(self._text)
        self.AddWidget(vsizer, 1)
        self._progress = controls.Label(parent, '')
        self.AddWidget(self._progress)
        self._message.SetFont(font)
        self._text.SetFont(font)
        self._progress.SetFont(font)
        if callable(transaction.cancel):
            self.cancel_callback = transaction.cancel
            self._cancel = controls.buttons.Bitmap(parent, Images.GetIcon('cancel', 'small'), onclick=self.on_cancel, border=False)
            self.AddWidget(self._cancel, expand=False, align=controls.sizers.LEFT, padding=4)
            self._cancel.SetFont(font)
        else:
            self._cancel = None
    
    def on_cancel(self, e):
        self.end()
        self.cancel_callback()
        
    def end(self):
        self.alive = False
        if self._cancel: self._cancel.Show(False)
        
    def Destroy(self):
        self._text.Destroy()
        self._message.Destroy()
        self._progress.Destroy()
        if self._cancel: self._cancel.Destroy()
        controls.sizers.HBox.Destroy(self)
        
    def update_progress(self, transaction):
        self._progress.SetLabel(str(transaction.progress) + '%')
        self._text.SetLabel(transaction.status)
    
    def update_status(self, transaction):
        self._text.SetLabel(transaction.status)


class Frame(Window.Frame, Window.ThreadedPanelMixin):
    def __init__(self, parent):
        Window.Frame.__init__(self, parent, 'Transfers', style=wx.FRAME_TOOL_WINDOW|wx.FRAME_FLOAT_ON_PARENT)
        Window.ThreadedPanelMixin.__init__(self)
        if 'pos' not in self.settings: self.CentreOnScreen()
        
        self.sw = scrolled.ScrolledPanel(self, -1)
        self.sw.SetMinSize((300, 200))
        self.sw.SetBackgroundColour(wx.Colour(255, 255, 255))
        self.SetBackground(Images.GetColor('panel', 'background'))
        
        self.clear = controls.buttons.Button(self, 'Clear', self.on_clear)
        
        font = Window.GetFont('small')

        self.clear.SetFont(font)
        
        vsizer = controls.sizers.VBox()
        vsizer.AddWidget(self.sw, 1)
        hsizer = controls.sizers.HBox()
        hsizer.AddWidget(self.clear, 0, expand=False, padding=4)
        vsizer.AddWidget(hsizer, 0)
        self.SetSizer(vsizer)
        
        self.vsizer = controls.sizers.VBox()
        self.sw.SetSizer(self.vsizer)
        self.sw.SetAutoLayout(True)
        self.sw.SetupScrolling()
        
        self.sw.SetScrollRate(0, 1)
        
        self._c = 0
        self._ctls = {}
        logger.observe(self)
        if self.settings.get('visible', False):
            wx.CallAfter(self.Show)
        
    def on_clear(self, e):
        for transaction, t in self._ctls.items():
            if not t.alive:
                del self._ctls[transaction]
                self.vsizer.Detach(t)
                t.Destroy()
        self.vsizer.Layout()
        self.sw.SetupScrolling()
        self.sw.Layout()
        self.sw.Refresh()
        
    @Window.synchronized
    def notify_begin(self, group, transaction):
        t = Transfer(self.sw, transaction)
        self.vsizer.AddWidget(t, padding=8)
        self.sw.SetupScrolling()
        self.sw.Layout()
        self._ctls[transaction] = t
        return t
        
    @Window.synchronized
    def notify_progress(self, group, transaction):
        t = self._ctls[transaction]
        t.update_progress(transaction)
        self.sw.Layout()
            
    @Window.synchronized
    def notify_status(self, group, transaction):
        t = self._ctls[transaction]
        t.update_status(transaction)
        self.sw.Layout()

    @Window.synchronized
    def notify_end(self, group, transaction, success):
        t = self._ctls[transaction]
        t.end()
        if success:
            del self._ctls[transaction]
            self.vsizer.Detach(t)
            t.Destroy()
        else:
            t.update_status(transaction)
        self.vsizer.Layout()
        self.sw.SetupScrolling()
        self.sw.Layout()
        self.sw.Refresh()
        
    def OnClose(self, e=None):
        self.SaveState()
        self.Show(False)
        if e and hasattr(e, 'Veto'): e.Veto()

    def MakeMenu(self):
        if wx.Platform == '__WXMSW__':
            Window.Frame.MakeMenu(self)
            
    def OnMenu(self, e):
        id = e.GetId()
        if id == self.ID_CLOSE:
            self.Close()

    def OnKey(self, e):
        if e.GetKeyCode() == wx.WXK_ESCAPE:
            self.Close()
        e.Skip()
        
    def GetDefaultConfig(self):
        c = Window.Frame.GetDefaultConfig(self)
        c.size = [400, 250]
        c.visible = True
        return c
        
    def GetCurrentConfig(self):
        c = Window.Frame.GetCurrentConfig(self)
        c.visible = self.IsShown()
        return c

