from __future__ import with_statement

import threading
import time
import posixpath

import AppInfo
from Config import Config
import auto_update
from lib import async, logger
import Images

from . import Window, controls, messages

def check(parent):
    when = Config.auto_update.get('when', 1)
    if when > 0:
        if when == 1:
            Frame(parent)
        elif when == 2:
            # weekly
            last_check = Config.auto_update.get('last_check', None)
            if last_check is None or last_check <= time.time() - 604800:
                Frame(parent)


class Frame(Window.Frame, Window.ThreadedPanelMixin):
    def __init__(self, parent):
        Window.Frame.__init__(self, parent, 'Update', resizable=False)
        Window.ThreadedPanelMixin.__init__(self)
        self.SetBackground(Images.GetColor('panel', 'background'))
        self.jobrunner = async.JobRunner(daemon=True)
        self.jobrunner.observe(Window.AsyncUpdateHandler(self))
        self.jobrunner.start()
        self.jobrunner.post(msg=logger.begin('Checking for updates', 'update.check'), method=self.CheckUpdate, on_complete=self.OnCheckComplete)
    
    def CheckUpdate(self, *args, **kwargs):
        u = auto_update.Updater(posixpath.join(AppInfo.website, 'update'), AppInfo.build_date)
        u.get_xml()
        return u
    
    def OnCheckComplete(self, updater):
        self.jobrunner.stop()
        Config.auto_update.last_check = time.time()
        Config.save()
        if updater.has_update():
            self.updater = updater
            info = updater.get_info()
            self.Populate(info)
            self.Show()
        else:
            self.Destroy()
    
    def OnAsyncError(self, e):
        Window.ThreadedPanelMixin.OnAsyncError(self, e)
        self.jobrunner.stop()
        self.Destroy()
        
    def Populate(self, info):
        self.SetTitle(info['title'])
        vsizer = controls.sizers.VBox()
        label = controls.Label(self, 'An update to version %s is now available, would you like to download it now?' % (info['version'],))
        vsizer.AddWidget(label, padding=8)
        label.Wrap(380)
        description = info.get('description', '')
        if len(description):
            html = controls.HtmlWindow(self)
            html.SetHtml("<html><body>" + description + "</body></html>")
            html.SetMinSize((300, 200))
            vsizer.AddWidget(html, padding=8)
        hsizer = controls.sizers.HBox()
        cancel = controls.buttons.Button(self, 'No', self.OnNo)
        ok = controls.buttons.Button(self, 'Yes', self.OnYes)
        hsizer.AddWidget(cancel, padding=4)
        hsizer.AddWidget(ok, padding=4)
        vsizer.AddWidget(hsizer, align=controls.sizers.ALIGN_RIGHT|controls.sizers.ALL, padding=8, expand=False)
        self.SetSizer(vsizer)
        vsizer.Fit(self)
        
    def OnNo(self, e):
        self.Close()
        self.Destroy()
    
    def OnYes(self, e):
        Downloader(self.Parent, self.updater).Show()
        self.Close()
        self.Destroy()


class Downloader(Window.Frame, Window.ThreadedPanelMixin):
    def __init__(self, parent, updater):
        Window.Frame.__init__(self, parent, 'Downloading Update', resizable=False)
        self.SetMinSize((300, 30))
        Window.ThreadedPanelMixin.__init__(self)
        self.SetBackground(Images.GetColor('panel', 'background'))
        self.updater = updater
        self.complete = False
        self._transfer = None
        self.filename = None
        self.jobrunner = async.JobRunner(daemon=True)
        self.jobrunner.observe(Window.AsyncUpdateHandler(self))
        self.jobrunner.start()
        self.jobrunner.post(method=self.DownloadUpdate, on_complete=self.OnDownloadComplete)
        vsizer = controls.sizers.VBox()
        vsizer.AddWidget(controls.Label(self, 'Downloading Update'), padding=8)
        self.progress = controls.Progress(self, 100)
        vsizer.AddWidget(self.progress, padding=8)
        hsizer = controls.sizers.HBox()
        self.button = controls.buttons.Button(self, 'Cancel', self.OnCancel)
        hsizer.AddWidget(self.button)
        vsizer.AddWidget(hsizer, align=controls.sizers.ALIGN_RIGHT|controls.sizers.ALL, padding=8, expand=False)
        self.SetSizer(vsizer)
        vsizer.Fit(self)
        
    def MakeMenu(self):
        pass
        
    def OnCancel(self, e):
        if self._transfer: self._transfer.error(Exception('Cancelled'))
        self.Close()
        if self.button.GetLabel() == 'Install and Relaunch' and self.filename is not None:
            auto_update.install(self.filename)
        
    def OnClose(self, e):
        self.complete = True
        self.jobrunner.stop()
        Window.Frame.OnClose(self, e)
        
    @Window.synchronized
    def on_progress(self, p, n):
        self._transfer.update_progress(p, n)
        self.progress.SetRange(n)
        self.progress.SetValue(p)
        
    def on_complete(self, filename):
        self.complete = True
        self.filename = filename
        if self._transfer: self._transfer.complete('Finished')
        
    def OnAsyncError(self, e):
        Window.ThreadedPanelMixin.OnAsyncError(self, e)
        messages.Error(self, 'Unable to download update, sorry!\nPlease try again later.')
        self.Close()
        
    def DownloadUpdate(self):
        with logger.begin('Downloading updates', 'update.download') as msg:
            self._transfer = msg
            self.updater.begin_update(self.on_progress, self.on_complete)
            while not self.complete:
                time.sleep(1)
            return self.filename
        
    def OnDownloadComplete(self, filename):
        self.jobrunner.stop()
        self.button.SetLabel('Install and Relaunch')
        self.GetSizer().Layout()
