
import os
import sys
import weakref

import wx

from Config import Config, ConfigDict
from lib import logger
import Images

import Threading
import controls
import Events


def GetFont(size='normal'):
    font = wx.SystemSettings_GetFont(wx.SYS_DEFAULT_GUI_FONT)
    if size == 'small':
        if wx.Platform == '__WXMSW__': font.SetPointSize(8)
        else: font.SetPointSize(11)
    elif size == 'large':
        if wx.Platform == '__WXMSW__': font.SetPointSize(14)
        else: font.SetPointSize(16)
    return font


CommandEvent = Events.CommandEvent
RunToolEvent = controls.Toolbar.RunToolEvent

_PT_EVENT = wx.NewEventType()

def EVT_TITLE_CHANGE(win, func):
    win.Connect(-1, -1, _PT_EVENT, func)

class PageTitleEvent(Events.CommandEvent):
    type = _PT_EVENT
    def getPage(self):
        return self.page
    def getTitle(self):
        return self.title


EVT_SETUP_COMPLETE, SetupCompleteEvent = Events.create('SetupComplete')
EVT_SYNC_METHOD, SyncEvent = Events.create('Sync')
EVT_STATUS_PUSH, StatusPushEvent = Events.create('StatusPush')
EVT_STATUS_POP, StatusPopEvent = Events.create('StatusPop')
EVT_STATUS_TEXT, StatusTextEvent = Events.create('StatusText')

def synchronized(fn):
    def run_synchronized(self, *args, **kwargs):
        def _(): fn(self, *args, **kwargs)
        self.PostEvent(SyncEvent(method=_))
    return run_synchronized


_PANEL_EVENT = wx.NewEventType()

def EVT_PANEL_CHANGE(win, func):
    win.Connect(-1, -1, _PANEL_EVENT, func)

class PanelChangeEvent(Events.CommandEvent):
    type = _PANEL_EVENT
    def __init__(self, klass, *args, **kwargs):
        self.klass = klass
        Events.CommandEvent.__init__(self, *args, **kwargs)
        if 'newtab' in self.kw: del self.kw['newtab']
    def WantNewTab(self):
        if hasattr(self, 'newtab'): return self.newtab
        return wx.GetKeyState(wx.WXK_CONTROL) or wx.GetKeyState(wx.WXK_MENU) or wx.GetKeyState(wx.WXK_ALT)
    def GetArgs(self):
        return self.args
    def GetKwargs(self):
        return self.kw
    def GetPanelClass(self):
        return self.klass
    def CreatePanel(self, parent):
        return self.GetPanelClass()(parent, *self.GetArgs(), **self.GetKwargs())


def ParseShortcutAccel(s, id):
    parts = s.replace('+', '-').split('-')
    k = parts.pop()
    m = 0
    for p in [p.lower() for p in parts]:
        if p == 'ctrl':
            if wx.Platform == '__WXMAC__': m |= wx.ACCEL_CMD
            else: m |= wx.ACCEL_CTRL
        elif p == 'alt': m |= wx.ACCEL_ALT
        elif p == 'shift': m |= wx.ACCEL_SHIFT
    return (m, ord(k), id)


class Panel(wx.PyPanel, Events.EventHandler):
    def __init__(self, parent, style=0):
        wx.PyPanel.__init__(self, parent, -1, style=wx.BORDER_NONE|style)
        EVT_SYNC_METHOD(self, self.OnSyncMethod)

    def SetBackground(self, colour):
        self.SetBackgroundStyle(wx.BG_STYLE_COLOUR)
        self.SetBackgroundColour(wx.Colour(*colour))
        
    def ChangePanel(self, PanelClass, *args, **kwargs):
        self.PostEvent(PanelChangeEvent(PanelClass, *args, **kwargs))

    def FindParent(self, klass):
        p = self
        while p is not None:
            if isinstance(p, klass):
                break
            p = p.GetParent()
        return p

    def DoClipboardCopy(self):
        # if self.FindFocus().GetParent() == self.mainctrl:
        data = self.GetClipboardData()
        if data != "" and data is not None and wx.TheClipboard.Open():
            if isinstance(data, basestring):
                data = wx.TextDataObject(data)
            wx.TheClipboard.SetData(data)
            wx.TheClipboard.Close()
            return True
            
    def OnSyncMethod(self, e):
        e.method()



class PanelWithoutFocus(Panel):
    def AcceptsFocusFromKeyboard(self):
        return False


class WindowConfigMixin(object):
    def __init__(self):
        self.LoadConfig()
    def LoadConfig(self):
        self.settings = Config.RestoreWindow(self, self.GetConfigId(), self.GetDefaultConfig())
    def GetConfigId(self):
        return self.__module__ + '.' + self.__class__.__name__
    def GetDefaultConfig(self):
        return ConfigDict()
    def GetCurrentConfig(self):
        return ConfigDict()
    def SaveState(self):
        try:
            Config.SaveWindow(self, self.GetConfigId(), ConfigDict(**self.GetCurrentConfig()))
            Config.save()
        except Exception, x:
            print str(x)
    def OnClose(self, e=None):
        self.SaveState()
        if e: e.Skip()


class Frame(wx.Frame, WindowConfigMixin, Events.EventHandler):
    ID_CLOSE = wx.NewId()
    def __init__(self, parent, title='', resizable=True, style=0):
        s = wx.SYSTEM_MENU | wx.CAPTION | wx.CLOSE_BOX | wx.CLIP_CHILDREN
        if resizable: s |= wx.MINIMIZE_BOX | wx.MAXIMIZE_BOX | wx.RESIZE_BORDER
        s |= style
        wx.Frame.__init__(self, parent, -1, title, style=s)
        WindowConfigMixin.__init__(self)
        if self.settings.get('pos', None) is None: self.CentreOnParent()
        self.SetupIcon()
        wx.EVT_CLOSE(self, self.OnClose)
        wx.EVT_KEY_UP(self, self.OnKey)
        EVT_SYNC_METHOD(self, self.OnSyncMethod)
        self.MakeMenu()
        
    def SetupIcon(self):
        if wx.Platform == '__WXMSW__':
            ib=wx.IconBundle()
            ib.AddIconFromFile(Images.GetIcon("msw-ico"), wx.BITMAP_TYPE_ANY)
            self.SetIcons(ib)
        
    def MakeMenu(self):
        menu_bar = wx.MenuBar()
        self.SetupMenu(menu_bar)
        self.SetMenuBar(menu_bar)
        
    def SetupMenu(self, menu_bar):
        menu = wx.Menu()
        menu.Append(self.ID_CLOSE, "&Close\tCtrl-W", "Close this window")
        menu_bar.Append(menu, '&File')
        wx.EVT_MENU(self, self.ID_CLOSE, self.OnMenu)
        
    def SetBackground(self, colour):
        self.SetBackgroundStyle(wx.BG_STYLE_COLOUR)
        self.SetBackgroundColour(wx.Colour(*colour))
        
    def OnSyncMethod(self, e):
        e.method()
        
    def OnMenu(self, e):
        id = e.GetId()
        if id == self.ID_CLOSE:
            self.Close()
            self.Destroy()

    def OnKey(self, e):
        if e.GetKeyCode() == wx.WXK_ESCAPE:
            self.Close()
            self.Destroy()
        e.Skip()
        
    def FindParent(self, klass):
        p = self
        while p is not None:
            if isinstance(p, klass):
                break
            p = p.GetParent()
        return p


class MiniFrame(wx.MiniFrame, WindowConfigMixin, Events.EventHandler):
    def __init__(self, parent, title='', stay_on_top=False):
        style = wx.DEFAULT_FRAME_STYLE
        if stay_on_top:
            style |= wx.STAY_ON_TOP
        wx.MiniFrame.__init__(self, parent, -1, title, style=style)
        self.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)

        WindowConfigMixin.__init__(self)
        if self.settings.get('pos', None) is None: self.CentreOnParent()

        wx.EVT_CLOSE(self, self.OnClose)
        wx.EVT_KEY_UP(self, self.OnKey)
        EVT_SYNC_METHOD(self, self.OnSyncMethod)

    def SetBackground(self, colour):
        self.SetBackgroundStyle(wx.BG_STYLE_COLOUR)
        self.SetBackgroundColour(wx.Colour(*colour))

    def OnSyncMethod(self, e):
        e.method()

    def OnKey(self, e):
        if e.GetKeyCode() == wx.WXK_ESCAPE:
            self.Close()
            self.Destroy()
        e.Skip()



class ConfiguredPanel(Panel, WindowConfigMixin):
    def __init__(self, parent, style=0):
        Panel.__init__(self, parent, style=style|wx.TAB_TRAVERSAL)
        WindowConfigMixin.__init__(self)
        
        self.SetBackgroundColour(wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOW))
        
        self.BeforeCreatePanel()
        
        self.mainctrl = self.CreatePanel(self)
        
        self.AfterCreatePanel()
        
        vsizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(vsizer)
        vsizer.Layout()
        
        self.Layout()
        
        wx.EVT_CLOSE(self, self.OnClose)
    
    def UpdateMenu(self, mb):
        pass

    def BeforeCreatePanel(self):
        pass

    def CreatePanel(self, parent):
        return wx.Panel(parent, -1)

    def AfterCreatePanel(self):
        pass

    def SetSizer(self, sizer):
        sizer.Add(self.mainctrl, 1, wx.EXPAND|wx.ALL, 0)
        Panel.SetSizer(self, sizer)

    def Layout(self):
        Panel.Layout(self)

    def OnClose(self, e):
        self.SaveState()
    
    def ReturnFocus(self):
        self.mainctrl.SetFocus()
        
    def Activate(self):
        self.ReturnFocus()


class ToolsMixin(object):
    def __init__(self):
        self.tb = None
        
    def BeforeCreatePanel(self):
        self.CreateToolbar()
        
    def AfterCreatePanel(self):
        self.SetupToolbar()
        self.InitToolbarState()
        
    def CreateToolbar(self):
        if self.tb is not None:
            self.tb.Destroy()
            self.tb = controls.Toolbar.Toolbar(self, textalign='bottom')
        else:
            self.tb = controls.Toolbar.Toolbar(self, textalign='bottom')
            controls.Toolbar.EVT_RUN_TOOL(self, self.OnRunTool)
        
    def SetupToolbar(self):
        pass
    def UpdateTools(self):
        pass
        
    def InitToolbarState(self):
        pass

    def SetSizer(self, sizer):
        sizer.Add(self.tb, 0, wx.EXPAND)

    def Layout(self):
        self.tb.DoLayout()

    def OnRunTool(self, e):
        kwargs = e.kw
        if 'tool' in kwargs: del kwargs['tool']
        self.RunTool(e.tool, *e.args, **kwargs)
    
    def RunTool(self, tool, *args, **kwargs):
        if isinstance(tool, basestring):
            tool = self.tb.GetHandler(tool)
        return tool(*args, **kwargs)


class PanelWithTools(ConfiguredPanel, ToolsMixin):
    def __init__(self, parent, style=0):
        ToolsMixin.__init__(self)
        ConfiguredPanel.__init__(self, parent)
        
    def BeforeCreatePanel(self):
        ToolsMixin.BeforeCreatePanel(self)

    def AfterCreatePanel(self):
        ToolsMixin.AfterCreatePanel(self)

    def SetSizer(self, sizer):
        ToolsMixin.SetSizer(self, sizer)
        ConfiguredPanel.SetSizer(self, sizer)

    def Layout(self):
        ToolsMixin.Layout(self)
        ConfiguredPanel.Layout(self)


class AsyncUpdateHandler(object):
    def __init__(self, panel):
        self.panel = weakref.ref(panel)
    def __call__(self, e):
        panel = self.panel()
        if panel is not None and panel:
            wx.PostEvent(panel, Threading.AsyncUpdateEvent(**e))
    def notify_async_update(self, *args, **kwargs):
        self(kwargs)

class ThreadedPanelMixin(object):
    def __init__(self):
        Threading.EVT_ASYNC_UPDATE(self, self.OnAsyncUpdate)
        
    def OnAsyncStart(self, e):
        if hasattr(self, 'throbber'):
            self.throbber.Start()
        e.log()

    def OnAsyncComplete(self, e):
        if e.execute() != 'nolog':
            e.log()
        if hasattr(self, 'throbber'):
            self.throbber.Rest()
        
    def OnAsyncError(self, e):
        if e.execute() != 'nolog':
            e.log()
        if hasattr(self, 'throbber'):
            self.throbber.Rest()

    def OnAsyncUpdate(self, e):
        if e.status == 'start':
            self.OnAsyncStart(e)
        elif e.status == 'error':
            self.OnAsyncError(e)
        else:
            self.OnAsyncComplete(e)


class Tool(object):
    name = ''
    label = ''
    icon = ''
    description = ''
    default_shortcut = ''
    can_toolbar = False
    can_menu = False
    uses_selection = False
    override = False
    def __init__(self, parent, **kwargs):
        self.__dict__.update(kwargs)
        self.parent = parent
        if self.icon == '': self.icon = self.name
        if hasattr(parent, 'PostJob'): self.PostJob = parent.PostJob
        if hasattr(parent, 'RunTool'): self.RunTool = parent.RunTool
        if self.can_toolbar:
            self.parent.tb.Add(self, self.name, self.label, Images.GetIcon(self.icon), self.description)
            self.parent.tb.Enable(self.name, self.IsAble())

    def IsAble(self):
        return True
        
    def run(self):
        pass
    
    def on_trigger(self, e):
        self.run()
    
    def __call__(self, *args, **kwargs):
        return self.run(*args, **kwargs)

class ToolSeparator(object):
    can_toolbar = True
    def __init__(self, parent):
        if self.can_toolbar:
            if len(parent.tb.tools):
                parent.tb.AddSeparator()
            else:
                self.can_toolbar = False
    def IsAble(self): return False
    def run(self): pass
    def on_trigger(self, e): self.run()
    def __call__(self, *args, **kwargs): return self.run(*args, **kwargs)

class Shell(wx.Frame):
    def __init__(self, parent, title, locals, size=None, introText=''):
        if size is None: size = (600, 300)
        wx.Frame.__init__(self, parent, -1, 'Shell', size=size)
        locals['exit'] = self.Close
        import wx.py as py
        self.shell = wx.py.shell.Shell(self, wx.NewId(), locals=locals, introText=introText)

def IterateListCtrl(ctrl):
    i = -1
    while 1:
        i = ctrl.GetNextItem(i, wx.LIST_NEXT_ALL, wx.LIST_STATE_DONTCARE)
        if i == -1:
            break
        yield i

def IterateListCtrlSelections(ctrl):
    i = ctrl.GetFirstSelected()
    while i > -1:
        yield i
        i = ctrl.GetNextSelected(i)

