
import wx
import wx.lib.mixins.listctrl as listmix

class ListFindTimer(wx.Timer):
    """
    Timer used to clear ListCtrl._findPrefix if no key was pressed
    for a sufficiently long time.
    """

    def __init__(self, owner):
        """
        Default class constructor.
        For internal use: do not call it in your code!
        """

        wx.Timer.__init__(self)
        self._owner = owner


    def Notify(self):
        """The timer has expired."""

        self._owner._findPrefix = ""

_DELAY = 500

class ListCtrl(wx.ListCtrl, listmix.ColumnSorterMixin):
    
    EVT_ITEM_SELECT = wx.EVT_LIST_ITEM_SELECTED
    EVT_ITEM_DESELECT = wx.EVT_LIST_ITEM_DESELECTED
    EVT_ITEM_DCLICK = wx.EVT_LIST_ITEM_ACTIVATED
    EVT_ITEM_RCLICK = wx.EVT_LIST_ITEM_RIGHT_CLICK
    EVT_COL_DRAGGING = wx.EVT_LIST_COL_DRAGGING
    EVT_COL_END_DRAG = wx.EVT_LIST_COL_END_DRAG
    EVT_FOCUS = wx.EVT_SET_FOCUS
    EVT_KEY_DOWN = wx.EVT_LIST_KEY_DOWN
    
    AUTOSIZE = wx.LIST_AUTOSIZE
    AUTOSIZE_HEADER = wx.LIST_AUTOSIZE_USEHEADER
    
    def __init__(self, parent, multiple_select=True, sort=True):
        self.itemDataMap = {}
        style = wx.LC_REPORT|wx.SUNKEN_BORDER
        if sort: style |= wx.LC_SORT_ASCENDING
        if not multiple_select:
            style |= wx.LC_SINGLE_SEL
        wx.ListCtrl.__init__(self, parent, -1, style=style)
        self._findPrefix = ''
        self._findTimer = None
        self.Setup()
        if sort:
            listmix.ColumnSorterMixin.__init__(self, self.GetColumnCount())
        self.On(self.EVT_KEY_DOWN, self.OnKeyDown)
        
    def Setup(self):
        pass
        
    def GetListCtrl(self):
        return self

    def On(self, event_id, callback):
        self.Bind(event_id, callback, self)

    def DeleteAllItems(self):
        self.itemDataMap = {}
        self.UnselectAll()
        wx.ListCtrl.DeleteAllItems(self)
        
    def InsertColumn(self, num, label):
        wx.ListCtrl.InsertColumn(self, num, label)
        listmix.ColumnSorterMixin.SetColumnCount(self, self.GetColumnCount())
        
    def GetSortColumn(self):
        return self._col
        
    def GetSortAscending(self):
        return self._colSortFlag[self._col]
    
    def Sort(self, column=None, ascending=True):
        if column is None:
            self.SortListItems()
        else:
            self.SortListItems(column, ascending)
        
    def SelectAll(self, sel=True):
        i = -1
        j = -2
        while 1:
            i = self.GetNextItem(i)
            if i == -1 or i <= j:
                break
            j = i
            if sel:
                self.SelectItem(i)
            else:
                self.UnselectItem(i)
    def SelectInverse(self):
        i = -1
        j = -2
        while 1:
            i = self.GetNextItem(i)
            if i == -1 or i <= j:
                break
            j = i
            if self.IsSelected(i):
                self.UnselectItem(i)
            else:
                self.SelectItem(i)
    
    def FocusItem(self, i):
        self.SetItemState(i, wx.LIST_STATE_FOCUSED, wx.LIST_STATE_FOCUSED)
    
    def IsSelected(self, i):
        return self.GetItemState(i, wx.LIST_STATE_SELECTED)
        
    def SelectItem(self, i):
        self.SetItemState(i, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
        
    def UnselectItem(self, i):
        self.SetItemState(i, 0, wx.LIST_STATE_SELECTED)
        
    def UnselectAll(self):
        for i in self.iterSelected():
            self.SetItemState(i, 0, wx.LIST_STATE_SELECTED)
            
    def iterItems(self):
        for id, item in self.itemDataMap.iteritems():
            yield item
            
    def iterSelected(self):
        i = -1
        j = -2
        while 1:
            i = self.GetNextSelected(i)
            if i == -1 or i <= j:
                break
            j = i
            yield i
    
    def iterSelectedData(self):
        i = -1
        j = -2
        while 1:
            i = self.GetNextSelected(i)
            if i == -1 or i <= j:
                break
            j = i
            yield self.itemDataMap[self.GetItemData(i)]

    def GetSelection(self):
        for i in self.iterSelected():
            return i
    
    def PostEvent(self, event):
        wx.PostEvent(self, event)

    def OnKeyDown(self, e):
        keyCode = e.GetKeyCode()
        HasModifiers = wx.GetKeyState(wx.WXK_ALT) or wx.GetKeyState(wx.WXK_CONTROL) or wx.GetKeyState(wx.WXK_COMMAND)

        # if not ke.HasModifiers() and ((keyCode >= ord('0') and keyCode <= ord('9')) or \
        if (not HasModifiers and (keyCode >= ord('0') and keyCode <= ord('9')) or \
                                         (keyCode >= ord('A') and keyCode <= ord('z'))):

            # find the next item starting with the given prefix
            ch = chr(keyCode)
            cur = self.GetSelection()
            if cur is None: cur = -1
            id = self.FindItem(cur, self._findPrefix + ch, True)
            if id == -1:
                id = self.FindItem(-1, self._findPrefix + ch, True)

            if id == -1:
                # no such item
                return

            self.UnselectAll()
            self.FocusItem(id)
            self.SelectItem(id)
            self.EnsureVisible(id)
            self._findPrefix += ch

            # also start the timer to reset the current prefix if the user
            # doesn't press any more alnum keys soon -- we wouldn't want
            # to use this prefix for a new item search
            if not self._findTimer:
                self._findTimer = ListFindTimer(self)

            self._findTimer.Start(_DELAY, wx.TIMER_ONE_SHOT)

        else:
            e.Skip()
