import wx

from Window import WindowConfigMixin
from . import Events
from . import controls

OK = wx.ID_OK
CANCEL = wx.ID_CANCEL

def NewId():
    return wx.NewId()

class DialogBase(wx.Dialog, WindowConfigMixin, Events.EventHandler):
    def __init__(self, parent, title, resizable=False):
        style = wx.DEFAULT_DIALOG_STYLE
        if resizable: style |= wx.RESIZE_BORDER
        wx.Dialog.__init__(self, parent, -1, title, size=self.GetDefaultSize(), style=style)
        WindowConfigMixin.__init__(self)
        if 'pos' not in self.settings:
            self.CenterOnParent()
        wx.EVT_CLOSE(self, self.OnClose)
        vsizer = controls.sizers.VBox()
        vsizer.AddWidget(self.CreatePanel(), 1, padding=8)
        buttons = self.GetButtons()
        if len(buttons):
            vsizer.AddWidget(self.CreateButtons(buttons), expand=False, align=controls.sizers.TOP|controls.sizers.BOTTOM|controls.sizers.RIGHT|controls.sizers.ALIGN_RIGHT, padding=8)
        self.SetSizer(vsizer)
        if not resizable or 'pos' not in self.settings:
            vsizer.Fit(self)
        
    def CreatePanel(self):
        pass
        
    def OnClose(self, e=None):
        self.EndModal(CANCEL)
        self.SaveState()
        
    def OnOK(self, e=None):
        self.EndModal(OK)
        self.SaveState()
        
    def GetDefaultSize(self):
        return (400, 200)
        
    def GetButtons(self):
        return (
            {'id': CANCEL, 'label': 'Cancel', 'onclick': self.OnClose},
            {'id': OK, 'label': 'OK', 'onclick': self.OnOK, 'default': True}
        )
        
    def CreateButtons(self, buttons):
        sizer = controls.sizers.HBox()
        for b in buttons:
            id = b['id']
            btn = wx.Button(self, id, b['label'])
            if 'default' in b and b['default']:
                btn.SetDefault()
            sizer.AddWidget(btn, padding=4)
            wx.EVT_BUTTON(self, btn.GetId(), b['onclick'])
        return sizer
    
    def NewId(self): return NewId()
    
    def FindParent(self, klass):
        p = self
        while p is not None:
            if isinstance(p, klass):
                break
            p = p.GetParent()
        return p


def FileOpen(parent, message='', wildcards=[('All Files', '*.*')], directory='', filename='', multiple=False):
    style = wx.OPEN
    if multiple: style |= wx.MULTIPLE
    dlg = wx.FileDialog(
        parent, message=message,
        defaultDir=directory, 
        defaultFile=filename,
        wildcard='|'.join(["%s|%s" % w for w in wildcards]),
        style=style
    )
    if dlg.ShowModal() == wx.ID_OK:
        if multiple:
            return dlg.GetPaths()
        return dlg.GetPath()
    return None


def FileSave(parent, message='', wildcards=[('All Files', '*.*')], directory='', filename=''):
    dlg = wx.FileDialog(
        parent, message=message,
        defaultDir=directory, 
        defaultFile=filename,
        wildcard='|'.join(["%s|%s" % w for w in wildcards]),
        style=wx.SAVE|wx.FD_OVERWRITE_PROMPT
    )
    if dlg.ShowModal() == wx.ID_OK:
        return dlg.GetPath()
    return None


def ChooseFolder(parent, message='Choose a directory:', directory=''):
    dlg = wx.DirDialog(parent, message=message,
        defaultPath=directory,
        style=wx.DD_DEFAULT_STYLE
        | wx.DD_DIR_MUST_EXIST
        | wx.DD_NEW_DIR_BUTTON
    )
    if dlg.ShowModal() == wx.ID_OK:
        return dlg.GetPath()
    return None


def TextEntry(parent, message='', title=''):
    d = wx.TextEntryDialog(parent, message, title)
    if wx.ID_OK == d.ShowModal():
        return d.GetValue()
    return None
